"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = runPrepareForPublishTask;

var _tmpPromise = _interopRequireDefault(require("tmp-promise"));

var _listr = _interopRequireDefault(require("listr"));

var _path = _interopRequireDefault(require("path"));

var _fsExtra = require("fs-extra");

var _toolkit = require("@aragon/toolkit");

var _apm = _interopRequireDefault(require("@aragon/apm"));

var _listrOptions = _interopRequireDefault(require("../../../helpers/listr-options"));

var _askToConfirm = _interopRequireDefault(require("../../../lib/publish/askToConfirm"));

var _preprareFiles = require("../../../lib/publish/preprareFiles");

var _generateArtifact = require("../../../lib/apm/generateArtifact");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

//

/**
 * ctx mandatory output
 * - pathToPublish {string}
 * @return {TaskList} Tasks
 */
function runPrepareForPublishTask({
  // Globals
  cwd,
  web3,
  module: arapp,
  apm: apmOptions,
  silent,
  debug,
  // Arguments
  /// Files
  publishDir,
  files,
  ignore,
  /// Http
  httpServedFrom,
  /// Storage
  provider,
  /// Conditionals
  onlyArtifacts,
  onlyContent,
  http,
  // Context
  initialRepo,
  initialVersion,
  version,
  contractAddress
}) {
  const apm = (0, _apm.default)(web3, apmOptions);
  return new _listr.default([{
    title: 'Prepare files for publishing',
    enabled: () => !http,
    task: async ctx => {
      // If no publishDir, create temporary directory
      const pathToPublish = publishDir || (await _tmpPromise.default.dir()).path;
      await (0, _preprareFiles.prepareFilesForPublishing)(pathToPublish, files, ignore); // (TODO): ctx mutation

      ctx.pathToPublish = pathToPublish;
      return `Files copied to temporary directory: ${pathToPublish}`;
    }
  }, {
    title: 'Check for --http-served-from argument and copy manifest.json to destination',
    enabled: () => http,
    task: async ctx => {
      if (!httpServedFrom) throw new Error('You need to provide --http-served-from argument');
      (0, _generateArtifact.changeManifestForHttpServedFrom)(httpServedFrom); // (TODO): ctx mutation

      ctx.pathToPublish = httpServedFrom;
    }
  }, {
    title: 'Generate application artifact',
    skip: () => onlyContent && !arapp.path,
    task: async (ctx, task) => {
      const outputPath = onlyArtifacts ? cwd : ctx.pathToPublish;
      const contractPath = arapp.path;
      const roles = arapp.roles;
      /**
       * Define `performArtifcatGeneration` and `invokeArtifactGeneration`
       * previously so they can be reused below depending on what question
       * has to be asked to the user.
       * NOTE: `listr-input` (wrapped in askToConfirm) requires to be
       * returned in order to work, so it's necessary to nest callbacks
       *
       * TODO: (Gabi) Use inquier to handle confirmation
       */

      async function performArtifcatGeneration(artifact) {
        (0, _generateArtifact.writeApplicationArtifact)(artifact, outputPath);
        await (0, _generateArtifact.generateFlattenedCode)(outputPath, contractPath);
        return `Saved artifact in ${outputPath}`;
      }

      async function invokeArtifactGeneration() {
        const contractInterfacePath = _path.default.resolve(cwd, 'build/contracts', _path.default.basename(contractPath, '.sol') + '.json');

        const contractInterface = (0, _fsExtra.readJsonSync)(contractInterfacePath);
        const sourceCode = await (0, _fsExtra.readFile)(contractPath, 'utf8');
        const artifact = await (0, _toolkit.generateApplicationArtifact)(arapp, contractInterface.abi, sourceCode);
        performArtifcatGeneration(artifact);
      } // If an artifact file exist we check it to reuse


      if ((0, _generateArtifact.artifactExists)(outputPath)) {
        const currentArtifactIsIdentical = !(0, _generateArtifact.checkIfNewArticatIsIdentical)(cwd, roles, contractPath, outputPath);

        if (!currentArtifactIsIdentical) {
          return (0, _askToConfirm.default)("Couldn't reuse artifact due to mismatches, regenerate now?", invokeArtifactGeneration);
        } else {
          return task.skip('Using existing artifact');
        }
      } // If only content we fetch artifacts from previous version


      if (onlyContent & !_toolkit.APM_INITIAL_VERSIONS.includes(version)) {
        try {
          task.output = 'Fetching artifacts from previous version';
          await (0, _generateArtifact.copyCurrentApplicationArtifacts)(web3, cwd, outputPath, initialRepo, version, roles, contractPath, apmOptions);

          if (!(0, _generateArtifact.flattenCodeFileExists)(outputPath)) {
            await (0, _generateArtifact.generateFlattenedCode)(outputPath, contractPath);
          }

          return task.skip(`Using artifacts from v${initialVersion}`);
        } catch (e) {
          if (e.message === 'Artifact mismatch') {
            return (0, _askToConfirm.default)("Couldn't reuse existing artifact due to mismatches, regenerate now?", invokeArtifactGeneration);
          } else {
            return (0, _askToConfirm.default)("Couldn't fetch current artifact version to copy it. Please make sure your IPFS or HTTP server are running. Otherwise, generate now?", invokeArtifactGeneration);
          }
        }
      }

      return invokeArtifactGeneration();
    }
  }, {
    title: `Publish intent`,
    task: async (ctx, task) => {
      ctx.contractInstance = null; // clean up deploy sub-command artifacts

      const accounts = await web3.eth.getAccounts();
      const from = accounts[0];
      ctx.intent = await apm.publishVersionIntent(from, arapp.appName, version, http ? 'http' : provider, http || ctx.pathToPublish, contractAddress);
    }
  }], (0, _listrOptions.default)(silent, debug));
}
//# sourceMappingURL=runPrepareForPublishTask.js.map