"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = runSetupTask;

var _listr = _interopRequireDefault(require("listr"));

var _web3Utils = require("web3-utils");

var _toolkit = require("@aragon/toolkit");

var _util = require("../../../util");

var _truffleRunner = require("../../../helpers/truffle-runner");

var _listrOptions = _interopRequireDefault(require("../../../helpers/listr-options"));

var _semver = _interopRequireDefault(require("semver"));

var _apm = _interopRequireDefault(require("@aragon/apm"));

var _deploy = require("../../deploy");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// helpers
// cmds
const getMajor = version => version.split('.')[0];
/**
 * @typedef {Object} VersionAppInfo
 * @property {string} contractAddress "0xaabb1122"
 * @property {string} version "0.2.4"
 * // FUN! may include:
 * { error, contentURI }
 * or
 * { manifest, module, { content: { provider, location } }
 * or
 * { content: { provider, location } }
 *
 * // Where: if contentURI = "ipfs:Qmazz"
 * - provider: "ipfs"
 * - location: "Qmazz"
 */

/**
 * ctx mandatory output:
 * - initialRepo {VersionAppInfo}
 * - initialVersion {string} Version before release: "0.2.4"
 * - version {string} To release version: "0.2.5"
 * @return {TaskList} Tasks
 */


async function runSetupTask({
  // Globals
  gasPrice,
  cwd,
  web3,
  network,
  module,
  apm: apmOptions,
  silent,
  debug,
  // Arguments
  /// Scritps
  prepublish,
  prepublishScript,
  build,
  buildScript,
  /// Version
  bump,
  /// Contract
  contract,
  init,
  reuse,
  /// Conditionals
  onlyContent,
  onlyArtifacts,
  http
}) {
  if (onlyContent) {
    contract = _toolkit.ZERO_ADDRESS;
  }

  const apm = (0, _apm.default)(web3, apmOptions);
  return new _listr.default([{
    title: 'Start IPFS',
    skip: async () => (0, _toolkit.isLocalDaemonRunning)(),
    task: async () => {
      await (0, _toolkit.startLocalDaemon)((0, _toolkit.getBinaryPath)(), (0, _toolkit.getDefaultRepoPath)(), {
        detached: false
      });
    }
  }, {
    // TODO: During test decide which prepublish script choose here and for building
    title: 'Running prepublish script',
    enabled: () => prepublish,
    task: async (_0, task) => {
      const abortReason = await (0, _util.runScriptHelper)(prepublishScript, log => {
        task.output = log;
      });
      if (abortReason) task.skip(abortReason);
    }
  }, {
    title: `Applying version bump (${bump})`,
    task: async (ctx, task) => {
      let isValid = true;

      try {
        const ipfsTimeout = 1000 * 60 * 5; // 5min

        task.output = 'Fetching latest version from aragonPM...';
        ctx.initialRepo = await apm.getLatestVersion(module.appName, ipfsTimeout);
        ctx.initialVersion = ctx.initialRepo.version;
        ctx.version = _semver.default.valid(bump) ? _semver.default.valid(bump) : _semver.default.inc(ctx.initialVersion, bump);
        isValid = await apm.isValidBump(module.appName, ctx.initialVersion, ctx.version);

        if (!isValid) {
          throw new Error("Version bump is not valid, you have to respect APM's versioning policy. Check the version upgrade rules in the documentation: https://hack.aragon.org/docs/apm-ref.html#version-upgrade-rules");
        }

        ctx.shouldDeployContract = getMajor(ctx.initialVersion) !== getMajor(ctx.version);
      } catch (e) {
        if (e.message.indexOf('Invalid content URI') === 0) {
          return;
        } // Repo doesn't exist yet, deploy the first version


        ctx.version = _semver.default.valid(bump) ? _semver.default.valid(bump) : _semver.default.inc('0.0.0', bump); // All valid initial versions are a version bump from 0.0.0

        if (apm.validInitialVersions.indexOf(ctx.version) === -1) {
          throw new Error(`Invalid initial version  (${ctx.version}). It can only be 0.0.1, 0.1.0 or 1.0.0.`);
        }

        ctx.shouldDeployContract = true; // assume first version should deploy a contract
      }
    }
  }, {
    title: 'Building frontend',
    enabled: () => build && !http,
    task: async (_0, task) => {
      const abortReason = await (0, _util.runScriptHelper)(buildScript, log => {
        task.output = log;
      });
      if (abortReason) task.skip(abortReason);
    }
  }, {
    title: 'Compile contracts',
    enabled: () => !onlyContent && (0, _web3Utils.isAddress)(contract),
    task: async () => (0, _truffleRunner.compileContracts)()
  }, {
    title: 'Deploy contract',
    enabled: ctx => !onlyContent && (contract && !(0, _web3Utils.isAddress)(contract) || !contract && ctx.shouldDeployContract && !reuse),
    task: async ctx => {
      const deployTaskParams = {
        module,
        contract,
        init,
        gasPrice,
        network,
        cwd,
        web3,
        apmOptions
      };
      return (0, _deploy.task)(deployTaskParams);
    }
  }, {
    title: 'Determine contract address for version',
    enabled: () => !onlyArtifacts,
    task: async (ctx, task) => {
      // Get address of deployed contract
      ctx.contract = ctx.contractAddress;

      if ((0, _web3Utils.isAddress)(contract)) {
        ctx.contract = contract;
      } // Check if we can fall back to a previous contract address


      if (!ctx.contract && apm.validInitialVersions.indexOf(ctx.version) === -1) {
        task.output = 'No contract address provided, using previous one';

        try {
          const {
            contractAddress
          } = await apm.getLatestVersion(module.appName);
          ctx.contract = contractAddress;
          return `Using ${ctx.contract}`;
        } catch (err) {
          throw new Error('Could not determine previous contract');
        }
      } // Contract address required for initial version


      if (!ctx.contract) {
        throw new Error('No contract address supplied for initial version');
      }

      return `Using ${ctx.contract}`;
    }
  }], (0, _listrOptions.default)(silent, debug));
}
//# sourceMappingURL=runSetupTask.js.map