"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handler = exports.builder = exports.describe = exports.command = void 0;

var _chalk = require("chalk");

var _listr = _interopRequireDefault(require("listr"));

var _lodash = require("lodash");

var _cliTable = _interopRequireDefault(require("cli-table"));

var _web3Utils = require("web3-utils");

require("@aragon/toolkit/@types/acl/typedef");

require("@aragon/toolkit/@types/typedef");

var _toolkit = require("@aragon/toolkit");

var _daoArg = _interopRequireDefault(require("../utils/daoArg"));

var _knownApps = require("../utils/knownApps");

var _web3Fallback = require("../../../helpers/web3-fallback");

var _listrOptions = _interopRequireDefault(require("../../../helpers/listr-options"));

var _knownRoles = _interopRequireDefault(require("../../../knownRoles.json"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Load JSDoc types ACL specific
// Load JSDoc generic types
//
const command = 'view <dao>';
exports.command = command;
const describe = 'Inspect permissions in a DAO';
exports.describe = describe;

const builder = function (yargs) {
  return (0, _daoArg.default)(yargs);
};
/**
 * Return a task list for viewing DAO ACL permissions
 *
 * @param  {Object} args From Listr
 * @param  {string} args.dao DAO address or ENS name
 * @param  {NetworkConfig} args.network Network config
 * @param  {ApmConfig} args.apm APM config
 * @param  {WebsocketProvider} args.wsProvider Web3 config
 * @param  {ArappConfig} args.module arapp.json content
 * @param  {boolean} args.silent Silent flag
 * @param  {boolean} args.debug Debug flag
 * @return {Promise<TaskList>} void
 */


exports.builder = builder;

const handler = async function ({
  dao,
  network,
  apm,
  wsProvider,
  module,
  silent,
  debug
}) {
  const web3 = await (0, _web3Fallback.ensureWeb3)(network); // Type common context

  /**
   * @type {AclPermissions}
   */

  let permissions;
  /**
   * @type {App[]}
   */

  let apps;
  const tasks = new _listr.default([{
    title: 'Inspecting DAO Permissions',
    task: async (_, task) => {
      task.output = `Fetching permissions for ${dao}...`;
      const daoData = await (0, _toolkit.getDaoAddressPermissionsApps)({
        dao,
        web3Provider: wsProvider || web3.currentProvider,
        ipfsConf: apm.ipfs,
        apm
      });
      permissions = daoData.permissions;
      apps = daoData.apps;
      task.title = `Inspected DAO Permissions of ${daoData.daoAddress}`;
    }
  }], (0, _listrOptions.default)(silent, debug));
  return tasks.run().then(() => {
    const knownApps = (0, _knownApps.listApps)(module ? [module.appName] : []);
    const knownRoles = getKnownRoles(module);
    /**
     * @type {AclPermissionFormatted[]} Force type acknowledgment
     */

    const formatedAclPermissions = (0, _toolkit.formatAclPermissions)(permissions, apps, knownApps, knownRoles); // filter according to cli params will happen here

    const table = new _cliTable.default({
      head: ['App', 'Action', 'Allowed entities', 'Manager'].map(x => (0, _chalk.white)(x))
    });

    for (const formatedAclPermission of formatedAclPermissions) {
      const {
        to,
        manager,
        allowedEntities,
        role
      } = formatedAclPermission; // Ignore permissions with empty manager or allowed

      if (manager === _toolkit.ZERO_ADDRESS || !(allowedEntities || []).length) {
        continue;
      } // 1 - To


      const toFormatted = printNameOrAddress(to); // 2 - Role

      const roleFormatted = role.id || shortHex(role.hash); // 3 - Allowed entities (multiline)

      const allowedFormatted = allowedEntities.map(allowedEntity => (allowedEntity.address || '').toLowerCase() === _toolkit.ANY_ENTITY ? `🆓  Any entity` : `✅  ${printNameOrAddress(allowedEntity)}`).join('\n'); // 4 - Manager

      const managerFormatted = manager.address === _toolkit.NO_MANAGER ? `🆓  No manager` : printNameOrAddress(manager);
      table.push([toFormatted, roleFormatted, allowedFormatted, managerFormatted]);
    }

    console.log('\n', table.toString());
  });
};
/**
 * Helper to short hex string for better readability
 * @param  {string} hex
 *         '0x76804359e7b668845d209f4a0391d5482a18c476'
 * @return {string}
 *         '0x768043..18c476'
 */


exports.handler = handler;

const shortHex = hex => {
  return `${hex.slice(0, 8)}..${hex.slice(-6)}`;
};
/**
 * Helper to print out the short name of an app or its short address
 * @param  {AclPermissionAppInfo} appInfo App info
 * @return {string}
 *         'acl (0x5a10)' or '0x768043..18c476'
 */


const printNameOrAddress = ({
  address,
  name
}) => name ? `${name.split('.')[0]} (${address.slice(0, 6)})` : `${shortHex(address)}`;
/**
 * Returns this app roles and default known roles
 *
 * TODO: add support for user apps
 * @param {ArappConfig} module arapp.json contents
 * @return {Object.<string, RoleDefinition>} Unique known roles
 */


const getKnownRoles = module => {
  const currentAppRoles = module ? module.roles : [];

  const allRoles = _knownRoles.default.concat(currentAppRoles);

  return (0, _lodash.keyBy)(allRoles, role => (0, _web3Utils.keccak256)(role.id));
};
//# sourceMappingURL=view.js.map