"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handler = exports.builder = exports.describe = exports.command = void 0;

var _listr = _interopRequireDefault(require("listr"));

var _cliTable = _interopRequireDefault(require("cli-table"));

var _chalk = require("chalk");

var _connect = require("@aragon/connect");

var _web = _interopRequireDefault(require("web3"));

var _toolkit = require("@aragon/toolkit");

var _web3Fallback = require("../../helpers/web3-fallback");

var _listrOptions = _interopRequireDefault(require("../../helpers/listr-options"));

var _daoArg = _interopRequireDefault(require("./utils/daoArg"));

var _knownApps = require("./utils/knownApps");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

//
let knownApps;
const command = 'apps <dao>';
exports.command = command;
const describe = 'Get all the apps in a DAO';
exports.describe = describe;

const builder = function (yargs) {
  return (0, _daoArg.default)(yargs).option('all', {
    description: 'Whether to include apps without permissions as well',
    boolean: true
  });
};

exports.builder = builder;

const printAppNameFromAppId = appId => {
  return knownApps[appId] ? (0, _chalk.blue)(knownApps[appId]) : appId;
};

const printAppNameAndVersion = (appName, version) => {
  return version ? (0, _chalk.blue)(`${appName}@v${version}`) : (0, _chalk.blue)(appName);
};

const printContent = content => {
  if (!content) {
    return '(No UI available)';
  }

  return typeof content === 'string' ? content : `${content.provider}:${content.location}`;
};

const printApps = apps => {
  const appsContent = apps.map(({
    appId,
    proxyAddress,
    content,
    appName,
    version
  }) => [appName ? printAppNameAndVersion(appName, version) : printAppNameFromAppId(appId), proxyAddress, printContent(content)]).map(row => {
    row[0] = row[0].replace('.aragonpm.eth', '');
    return row;
  });
  const table = new _cliTable.default({
    head: ['App', 'Proxy address', 'Content'].map(x => (0, _chalk.white)(x))
  });
  appsContent.forEach(row => table.push(row));
  console.log(table.toString());
};

const printPermissionlessApps = apps => {
  if (apps && apps.length) {
    const tableForPermissionlessApps = new _cliTable.default({
      head: ['Permissionless app', 'Proxy address'].map(x => (0, _chalk.white)(x))
    });
    apps.forEach(app => tableForPermissionlessApps.push([printAppNameFromAppId(app.appId).replace('.aragonpm.eth', ''), app.proxyAddress]));
    console.log(tableForPermissionlessApps.toString());
  }
};
/**
 * Returns true if the provided Ethereum network id supports
 * Aragon Connect. Currently `true` for mainnet and Rinkeby
 * @param {number} Ethereum network id
 */


const supportsAragonConnect = networkId => {
  return networkId === 1 || networkId === 4;
};

const getAppsFromAragonConnect = async (dao, networkId) => {
  const org = await (0, _connect.connect)(dao, 'thegraph', {
    chainId: networkId
  });
  return (await org.apps()).map(app => ({
    appId: app.appId,
    appName: app.appName,
    proxyAddress: app.address,
    version: app.version,
    content: app.contentUri
  }));
};

const handler = async function ({
  reporter,
  dao,
  all,
  network,
  apm: apmOptions,
  wsProvider,
  module,
  silent,
  debug
}) {
  knownApps = (0, _knownApps.listApps)(module ? [module.appName] : []);
  const web3 = await (0, _web3Fallback.ensureWeb3)(network);
  let apps, daoAddress, appsWithoutPermissions;
  const tasks = new _listr.default([{
    title: 'Inspecting DAO',
    task: async (ctx, task) => {
      task.output = `Fetching apps for ${dao}...`;
      const {
        ensRegistryAddress,
        ipfs
      } = apmOptions;
      const options = {
        registryAddress: ensRegistryAddress,
        ipfs,
        provider: wsProvider || web3.currentProvider
      };
      daoAddress = await (0, _toolkit.getDaoAddress)(dao, options);
      apps = supportsAragonConnect(network.network_id) ? await getAppsFromAragonConnect(dao, network.network_id) : await (0, _toolkit.getInstalledApps)(dao, options);
    }
  }, {
    title: 'Fetching permissionless apps',
    enabled: () => all,
    task: async (ctx, task) => {
      appsWithoutPermissions = (await (0, _toolkit.getAllApps)(daoAddress, {
        web3: new _web.default(wsProvider || web3.currentProvider)
      })).filter(({
        proxyAddress
      }) => !apps.find(app => (0, _toolkit.addressesEqual)(app.proxyAddress, proxyAddress)));
    }
  }], (0, _listrOptions.default)(silent, debug));
  await tasks.run();
  reporter.newLine();
  reporter.success(`Successfully fetched DAO apps for ${(0, _chalk.green)(daoAddress)}`);
  printApps(apps);
  printPermissionlessApps(appsWithoutPermissions);
};

exports.handler = handler;
//# sourceMappingURL=apps.js.map