"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handler = exports.builder = exports.describe = exports.command = void 0;

var _listr = _interopRequireDefault(require("listr"));

var _chalk = require("chalk");

var _toolkit = require("@aragon/toolkit");

var _web3Fallback = require("../../helpers/web3-fallback");

var _listrOptions = _interopRequireDefault(require("../../helpers/listr-options"));

var _execHandler = require("./utils/execHandler");

var _daoArg = _interopRequireDefault(require("./utils/daoArg"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

//
const command = 'install <dao> <apmRepo> [apmRepoVersion]';
exports.command = command;
const describe = 'Install an app into a DAO';
exports.describe = describe;

const builder = function (yargs) {
  return (0, _daoArg.default)(yargs).option('apmRepo', {
    describe: 'Name of the aragonPM repo'
  }).option('apmRepoVersion', {
    describe: 'Version of the package upgrading to',
    default: 'latest'
  }).option('app-init', {
    description: 'Name of the function that will be called to initialize an app. Set it to "none" to skip initialization',
    default: 'initialize'
  }).option('app-init-args', {
    description: 'Arguments for calling the app init function',
    array: true,
    default: []
  }).options('set-permissions', {
    description: 'Whether to set permissions in the app. Set it to "open" to allow ANY_ENTITY on all roles.',
    choices: ['open']
  });
};

exports.builder = builder;

const handler = async function ({
  reporter,
  dao,
  gasPrice,
  network,
  apm: apmOptions,
  apmRepo,
  apmRepoVersion,
  appInit,
  appInitArgs,
  setPermissions,
  wsProvider,
  silent,
  debug
}) {
  const web3 = await (0, _web3Fallback.ensureWeb3)(network);
  const apmRepoName = (0, _toolkit.defaultAPMName)(apmRepo);
  dao = await (0, _toolkit.resolveAddressOrEnsDomain)(dao, web3, apmOptions.ensRegistryAddress);
  const tasks = new _listr.default([{
    title: 'Start IPFS',
    skip: async () => (0, _toolkit.isLocalDaemonRunning)(),
    task: async () => {
      await (0, _toolkit.startLocalDaemon)((0, _toolkit.getBinaryPath)(), (0, _toolkit.getDefaultRepoPath)(), {
        detached: false
      });
    }
  }, {
    title: `Fetching ${(0, _chalk.bold)(apmRepoName)}@${apmRepoVersion}`,
    task: async ctx => {
      const progressHandler = step => {
        switch (step) {
          case 1:
            console.log(`Initialize aragonPM`);
            break;

          case 2:
            console.log(`Fetching...`);
            break;
        }
      };

      ctx.repo = await (0, _toolkit.getApmRepo)(web3, apmRepoName, apmOptions, apmRepoVersion, progressHandler);
    }
  }, {
    title: `Checking installed version`,
    task: async (ctx, task) => {
      const currentBase = await (0, _toolkit.getAppBase)(dao, ctx.repo.appId, web3);

      if (currentBase === _toolkit.ZERO_ADDRESS) {
        task.skip(`Installing the first instance of ${apmRepoName} in DAO`);
        return;
      }

      if (!(0, _toolkit.addressesEqual)(currentBase, ctx.repo.contractAddress)) {
        throw new Error(`Cannot install app on a different version. Currently installed version for ${apmRepoName} in the DAO is ${currentBase}\n Please upgrade using 'dao upgrade' first or install a different version.`);
      }
    }
  }, {
    title: 'Deploying app instance',
    task: async ctx => {
      const initPayload = (0, _toolkit.encodeInitPayload)(web3, ctx.repo.abi, appInit, appInitArgs);

      if (initPayload === '0x') {
        ctx.notInitialized = true;
      }

      const fnArgs = [ctx.repo.appId, ctx.repo.contractAddress, initPayload, false];
      return (0, _execHandler.task)({
        dao,
        app: dao,
        method: 'newAppInstance(bytes32,address,bytes,bool)',
        // Use signature, method is overloaded
        params: fnArgs,
        reporter,
        gasPrice,
        apm: apmOptions,
        web3,
        wsProvider,
        silent,
        debug
      });
    }
  }, {
    title: 'Fetching deployed app',
    task: async (ctx, task) => {
      const appAddress = (0, _toolkit.getAppProxyAddressFromReceipt)(dao, ctx.receipt);
      if (appAddress) ctx.appAddress = appAddress;else task.skip("App wasn't deployed in transaction.");
    }
  }, {
    title: 'Set permissions',
    enabled: ctx => setPermissions === 'open' && ctx.appAddress,
    task: async (ctx, task) => {
      if (!ctx.repo.roles || ctx.repo.roles.length === 0) {
        throw new Error('You have no roles defined in your arapp.json.\nThis is required for your app to be properly installed.\nSee https://hack.aragon.org/docs/cli-global-confg#the-arappjson-file for more information.');
      }

      const permissions = ctx.repo.roles.map(role => [_toolkit.ANY_ENTITY, ctx.appAddress, role.bytes, _toolkit.NO_MANAGER]);

      if (!ctx.accounts) {
        ctx.accounts = await web3.eth.getAccounts();
      }

      const aclAddress = await (0, _toolkit.getAclAddress)(dao, web3);
      return Promise.all(permissions.map(params => {
        return (0, _execHandler.task)({
          dao,
          app: aclAddress,
          method: 'createPermission',
          params,
          reporter,
          gasPrice,
          apm: apmOptions,
          web3,
          wsProvider,
          silent,
          debug
        }) // execTask returns a TaskList not a promise
        .then(tasks => tasks.run());
      }));
    }
  }], (0, _listrOptions.default)(silent, debug));
  return tasks.run().then(ctx => {
    reporter.newLine();
    reporter.info(`Successfully executed: "${(0, _chalk.blue)(ctx.transactionPath.description)}"`);
    reporter.newLine();

    if (ctx.appAddress) {
      reporter.success(`Installed ${(0, _chalk.blue)(apmRepoName)} at: ${(0, _chalk.green)(ctx.appAddress)}`);
    } else {
      reporter.warning('After the app instance is created, you will need to assign permissions to it for it appear as an app in the DAO.');
    }

    if (ctx.notInitialized) {
      reporter.newLine();
      reporter.warning('The app could not be initialized, check the --app-init flag. Functions protected behind the ACL will not work until the app is initialized.');
    }
  });
};

exports.handler = handler;
//# sourceMappingURL=install.js.map