"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handler = exports.printResetNotice = exports.printMnemonic = exports.printAccounts = exports.task = exports.builder = exports.describe = exports.command = void 0;

var _listr = _interopRequireDefault(require("listr"));

var _ncp = _interopRequireDefault(require("ncp"));

var _os = _interopRequireDefault(require("os"));

var _path = _interopRequireDefault(require("path"));

var _rimraf = _interopRequireDefault(require("rimraf"));

var _fsExtra = require("fs-extra");

var _fs = _interopRequireDefault(require("fs"));

var _web = _interopRequireDefault(require("web3"));

var _chalk = require("chalk");

var _util = require("util");

var _toolkit = require("@aragon/toolkit");

var _ganacheCore = _interopRequireDefault(require("ganache-core"));

var _listrOptions = _interopRequireDefault(require("../../helpers/listr-options"));

var _package = _interopRequireDefault(require("../../../package.json"));

var _status = require("./status");

var _constants = require("./utils/constants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

//
const command = 'start';
exports.command = command;
const describe = 'Open a test chain for development and pass arguments to ganache';
exports.describe = describe;

const builder = yargs => {
  return yargs.option('port', {
    description: 'The port to run the local chain on',
    default: 8545,
    alias: 'p'
  }).option('network-id', {
    description: 'Network id to connect with',
    alias: 'i'
  }).option('hardfork', {
    description: 'Allows to specify which hardfork should be used. Supported hardforks are byzantium, constantinople, petersburg, and istanbul (default).'
  }).option('block-time', {
    description: 'Specify blockTime in seconds for automatic mining',
    alias: 'b'
  }).option('gas-limit', {
    default: _constants.BLOCK_GAS_LIMIT,
    description: 'Block gas limit. Must be specified as a hex string',
    alias: 'l'
  }).option('reset', {
    type: 'boolean',
    default: false,
    description: 'Reset devchain to snapshot',
    alias: 'r'
  }).option('accounts', {
    default: 2,
    description: 'Number of accounts to print',
    alias: 'a'
  }).option('verbose', {
    default: false,
    type: 'boolean',
    description: 'Enable verbose devchain output'
  });
};

exports.builder = builder;

const task = async function ({
  port = 8545,
  networkId,
  hardfork = 'istanbul',
  blockTime,
  gasLimit = _constants.BLOCK_GAS_LIMIT,
  verbose = false,
  reset = false,
  showAccounts = 2,
  reporter,
  silent,
  debug
}) {
  const removeDir = (0, _util.promisify)(_rimraf.default);
  const recursiveCopy = (0, _util.promisify)(_ncp.default);

  const snapshotPath = _path.default.join(_os.default.homedir(), `.aragon/devchain-db-${_package.default.version}`);

  const tasks = new _listr.default([{
    title: 'Check devchain status',
    task: async ctx => {
      const task = await (0, _status.task)({
        port,
        reset,
        silent,
        debug
      });
      const {
        portTaken,
        processID
      } = await task.run();

      if (portTaken && !reset) {
        throw new Error(`Process with ID ${(0, _chalk.red)(processID)} already running at port ${(0, _chalk.blue)(port)}`);
      }
    }
  }, {
    title: 'Setting up a new chain from latest Aragon snapshot',
    task: async (ctx, task) => {
      await removeDir(snapshotPath);
      await (0, _fsExtra.mkdirp)(_path.default.resolve(snapshotPath, '..'));
      const snapshot = (0, _toolkit.getAragonGanacheFiles)();
      await recursiveCopy(snapshot, snapshotPath);
    },
    enabled: () => !_fs.default.existsSync(snapshotPath) || reset
  }, {
    title: 'Starting a local chain from snapshot',
    task: async (ctx, task) => {
      ctx.id = parseInt(networkId) || parseInt(1e8 * Math.random());
      const options = {
        network_id: ctx.id,
        hardfork,
        blockTime,
        gasLimit,
        mnemonic: _constants.MNEMONIC,
        db_path: snapshotPath,
        logger: verbose ? {
          log: reporter.info.bind(reporter)
        } : undefined,
        debug
      };

      const server = _ganacheCore.default.server(options);

      const listen = () => new Promise((resolve, reject) => {
        server.listen(port, err => {
          if (err) return reject(err);
          task.title = `Local chain started at port ${(0, _chalk.blue)(port)}\n`;
          resolve();
        });
      });

      await listen();
      ctx.web3 = new _web.default(new _web.default.providers.WebsocketProvider(`ws://localhost:${port}`));
      const accounts = await ctx.web3.eth.getAccounts();
      ctx.accounts = accounts.slice(0, parseInt(showAccounts));
      ctx.mnemonic = _constants.MNEMONIC;
      const ganacheAccounts = server.provider.manager.state.accounts;
      ctx.privateKeys = ctx.accounts.map(address => ({
        key: ganacheAccounts[address.toLowerCase()].secretKey.toString('hex'),
        address
      }));
    }
  }], (0, _listrOptions.default)(silent, debug));
  return tasks;
};

exports.task = task;

const printAccounts = (reporter, privateKeys) => {
  const firstAccountComment = '(account used to deploy DAOs, has more permissions)';
  const formattedAccounts = privateKeys.map(({
    address,
    key
  }, i) => `Address #${i + 1}:  ${(0, _chalk.green)(address)} ${i === 0 ? firstAccountComment : ''}\nPrivate key: ` + (0, _chalk.blue)(key) + '\n');
  reporter.info(`Here are some Ethereum accounts you can use.
  The first one will be used for all the actions the aragonCLI performs.
  You can use your favorite Ethereum provider or wallet to import their private keys.
  \n${formattedAccounts.join('\n')}`);
};

exports.printAccounts = printAccounts;

const printMnemonic = (reporter, mnemonic) => {
  reporter.info(`The accounts were generated from the following mnemonic phrase:\n${(0, _chalk.blue)(mnemonic)}\n`);
};

exports.printMnemonic = printMnemonic;

const printResetNotice = (reporter, reset) => {
  if (reset) {
    reporter.warning(`${(0, _chalk.yellow)('The devchain was reset, some steps need to be done to prevent issues:')}
    - Reset the application cache in Aragon Client by going to Settings -> Troubleshooting.
    - If using Metamask: switch to a different network, and then switch back to the 'Private Network' (this will clear the nonce cache and prevent errors when sending transactions)
  `);
  }
};

exports.printResetNotice = printResetNotice;

const handler = async ({
  reporter,
  port,
  networkId,
  blockTime,
  gasLimit,
  reset,
  verbose,
  accounts,
  silent,
  debug
}) => {
  const tasks = await task({
    port,
    networkId,
    blockTime,
    gasLimit,
    reset,
    verbose,
    reporter,
    showAccounts: accounts,
    silent,
    debug
  });
  const {
    privateKeys,
    id,
    mnemonic
  } = await tasks.run();
  printAccounts(reporter, privateKeys);
  printMnemonic(reporter, mnemonic);
  printResetNotice(reporter, reset);
  reporter.info('ENS instance deployed at:', (0, _chalk.green)('0x5f6f7e8cc7346a11ca2def8f827b7a0b612c56a1'), '\n');
  reporter.info(`Network Id: ${(0, _chalk.blue)(id)}`, '\n');
  reporter.info(`Devchain running at: ${(0, _chalk.blue)('http://localhost:' + port)}.`); // Patch to prevent calling the onFinishCommand hook

  await new Promise((resolve, reject) => {});
};

exports.handler = handler;
//# sourceMappingURL=start.js.map