"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handler = exports.builder = exports.describe = exports.command = void 0;

var _listr = _interopRequireDefault(require("listr"));

var _inquirer = _interopRequireDefault(require("inquirer"));

var _chalk = require("chalk");

var _toolkit = require("@aragon/toolkit");

var _listrOptions = _interopRequireDefault(require("../../helpers/listr-options"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

//
const command = 'install';
exports.command = command;
const describe = 'Download and install the go-ipfs binaries.';
exports.describe = describe;

const builder = yargs => yargs.option('dist-version', {
  description: 'The version of IPFS that will be installed',
  default: _toolkit.GO_IMPL_DIST_VERSION
}).option('dist-url', {
  description: 'The url from which to download IPFS',
  default: _toolkit.GO_IMPL_DIST_URL
}).option('local', {
  description: 'Whether to install IPFS as a project dependency',
  boolean: true,
  default: false
}).option('project-path', {
  description: 'The project path to be used when installing locally',
  default: process.cwd()
}).option('skip-confirmation', {
  description: 'Whether to skip the confirmation step',
  boolean: true,
  default: false
});

exports.builder = builder;

const runPrepareTask = ({
  silent,
  debug,
  local,
  projectPath
}) => {
  return new _listr.default([{
    title: 'Determine platform and architecture',
    task: ctx => {
      ctx.NODE_OS = (0, _toolkit.getPlatform)();
      ctx.NODE_ARCH = (0, _toolkit.getArch)();
    }
  }, {
    title: 'Determine golang distribution',
    task: ctx => {
      ctx.GO_OS = (0, _toolkit.getPlatformForGO)();
      ctx.GO_ARCH = (0, _toolkit.getArchForGO)();
    }
  }, {
    title: 'Determine location',
    task: async ctx => {
      if (local) {
        ctx.location = projectPath;

        if (!(0, _toolkit.isPackage)(ctx.location)) {
          throw new Error(`${(0, _chalk.red)(ctx.location)} does not have a ${(0, _chalk.red)('package.json')}. Did you wish to install IPFS globally?`);
        }
      } else {
        ctx.location = await (0, _toolkit.getGlobalPackagesLocation)();
      }
    }
  }], Object.assign({
    concurrent: true
  }, (0, _listrOptions.default)(silent, debug))).run();
};

const runInstallTask = ({
  silent,
  debug,
  local,
  location,
  distUrl,
  distVersion
}) => {
  return new _listr.default([{
    title: 'Install IPFS',
    task: async task => {
      await (0, _toolkit.installGoIpfs)(local, location, distVersion, distUrl, {
        logger: text => task.output = text
      });
    }
  }], (0, _listrOptions.default)(silent, debug)).run();
};

const handler = async argv => {
  const {
    debug,
    silent,
    distVersion,
    distUrl,
    local,
    projectPath,
    reporter,
    skipConfirmation
  } = argv;
  /**
   * Check if it's already installed
   */

  const existingBinaryLocation = local ? (0, _toolkit.getLocalBinary)('ipfs', projectPath) : (0, _toolkit.getGlobalBinary)('ipfs');

  if (existingBinaryLocation) {
    reporter.error('IPFS is already installed:', (0, _chalk.red)(existingBinaryLocation));
    reporter.newLine();
    const uninstallCommand = local ? 'aragon ipfs uninstall --local' : 'aragon ipfs uninstall';
    reporter.warning('To install a different version, you must first run:', (0, _chalk.yellow)(uninstallCommand));

    if (!local) {
      reporter.warning('To install IPFS in a project, use the --local flag:', (0, _chalk.yellow)('aragon ipfs install --local'));
    }

    throw Error();
  }
  /**
   * Prepare confirmation details
   */


  reporter.info('Preparing:');
  const {
    NODE_OS,
    NODE_ARCH,
    GO_OS,
    GO_ARCH,
    location
  } = await runPrepareTask({
    debug,
    silent,
    local,
    projectPath
  });
  const actualVersion = (0, _toolkit.cleanVersion)(distVersion);
  const distName = (0, _toolkit.getDistName)(actualVersion, GO_OS, GO_ARCH);
  reporter.newLine();
  reporter.info(`Platform & architecture: ${(0, _chalk.blue)(NODE_OS)}, ${(0, _chalk.blue)(NODE_ARCH)}`);
  reporter.info(`IPFS tarball: ${(0, _chalk.blue)(distName)}`);
  reporter.info(`IPFS distributions url: ${(0, _chalk.blue)(distUrl)}`);
  reporter.info(`NPM version: ${(0, _chalk.blue)(distVersion)}`);
  reporter.info(`Location: ${(0, _chalk.blue)(location)}`);
  /**
   * Confirm & install
   */

  reporter.newLine();

  if (!skipConfirmation) {
    const {
      confirmation
    } = await _inquirer.default.prompt([{
      type: 'confirm',
      name: 'confirmation',
      message: `Are you sure you want to ${(0, _chalk.green)('install IPFS')}?`
    }]);
    if (!confirmation) return;
  }

  await runInstallTask({
    debug,
    silent,
    distUrl,
    distVersion,
    local,
    location
  });
  reporter.newLine();
  reporter.success('Success!');
  reporter.info(`Try it out with: ${(0, _chalk.blue)(local ? 'npx ipfs version' : 'ipfs version')}`);
};

exports.handler = handler;
//# sourceMappingURL=install.js.map