"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handler = exports.runPropagateTask = exports.builder = exports.describe = exports.command = void 0;

var _listr = _interopRequireDefault(require("listr"));

var _chalk = require("chalk");

var _toolkit = require("@aragon/toolkit");

var _listrOptions = _interopRequireDefault(require("../../helpers/listr-options"));

var _util = require("../../util");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

//
const command = 'propagate [cid]';
exports.command = command;
const describe = 'Request the content and its links at several gateways, making the files more distributed within the network. Uses --ipfs-gateway.';
exports.describe = describe;

const builder = yargs => yargs.positional('cid', {
  description: 'A self-describing content-addressed identifier'
});

exports.builder = builder;

const runPropagateTask = ({
  cid,
  ipfsReader,
  silent,
  debug
}) => {
  return new _listr.default([{
    title: 'Start IPFS',
    skip: async () => (0, _toolkit.isLocalDaemonRunning)(),
    task: async () => {
      await (0, _toolkit.startLocalDaemon)((0, _toolkit.getBinaryPath)(), (0, _toolkit.getDefaultRepoPath)(), {
        detached: false
      });
    }
  }, {
    title: 'Validate CID',
    task: () => {
      if (!(0, _toolkit.isValidCID)(cid)) {
        throw new Error(`"${cid}" is not a valid content identifier.`);
      }
    }
  }, {
    title: 'Fetch the links',
    task: async (ctx, task) => {
      const handleProgress = (step, data) => {
        switch (step) {
          case 1:
            task.output = `Fetch DAG information for ${(0, _chalk.gray)(data)}`;
            break;

          case 2:
            task.output = `Parse DAG information for ${(0, _chalk.gray)(data)}`;
            break;
        }
      };

      ctx.data = await (0, _toolkit.getMerkleDAG)(ipfsReader, cid, {
        recursive: true,
        progressCallback: handleProgress
      });
    }
  }, {
    title: 'Query gateways',
    task: async (ctx, task) => {
      const handleProgress = (step, data) => {
        switch (step) {
          case 1:
            {
              const cid = (0, _chalk.gray)(data.cid);
              const succeeded = (0, _chalk.green)(data.succeeded);
              const failed = (0, _chalk.red)(data.failed);
              task.output = `Queried ${cid} at ${succeeded} gateways successfully, ${failed} failed.`;
              break;
            }
        }
      };

      ctx.CIDs = (0, _toolkit.extractCIDsFromMerkleDAG)(ctx.data, {
        recursive: true
      });
      ctx.result = await (0, _toolkit.propagateFiles)(ctx.CIDs, {
        progressCallback: handleProgress
      });
    }
  }], (0, _listrOptions.default)(silent, debug)).run();
};

exports.runPropagateTask = runPropagateTask;

const handler = async argv => {
  /**
   * Interactive input
   */
  let {
    cid
  } = argv;

  if (!cid) {
    cid = await (0, _util.askForInput)('Choose a content identifier');
  }

  const {
    reporter,
    apm,
    debug,
    silent
  } = argv;
  const ipfsReader = await (0, _toolkit.getHttpClient)(apm.ipfs.gateway);
  const ctx = await runPropagateTask({
    ipfsReader,
    cid,
    debug,
    silent
  });
  reporter.message('\n', `Queried ${(0, _chalk.blue)(ctx.CIDs.length)} CIDs at ${(0, _chalk.blue)(ctx.result.gateways.length)} gateways`, '\n', `Requests succeeded: ${(0, _chalk.green)(ctx.result.succeeded)}`, '\n', `Requests failed: ${(0, _chalk.red)(ctx.result.failed)}`, '\n');
  reporter.debug(`Gateways: ${ctx.result.gateways.join(', ')}`);

  if (ctx.result.errors && ctx.result.errors.length) {
    reporter.debug(`Errors: \n${ctx.result.errors.map(JSON.stringify).join('\n')}`);
  } // TODO add your own gateways

};

exports.handler = handler;
//# sourceMappingURL=propagate.js.map