"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handler = exports.builder = exports.describe = exports.command = void 0;

var _listr = _interopRequireDefault(require("listr"));

var _publicIp = _interopRequireDefault(require("public-ip"));

var _internalIp = _interopRequireDefault(require("internal-ip"));

var _fs = require("fs");

var _chalk = require("chalk");

var _toolkit = require("@aragon/toolkit");

var _listrOptions = _interopRequireDefault(require("../../helpers/listr-options"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

//
const command = 'status';
exports.command = command;
const describe = 'Show whether the daemon is running and other useful information.';
exports.describe = describe;

const builder = yargs => yargs.option('repo-path', {
  description: 'The location of the IPFS repo',
  default: (0, _toolkit.getDefaultRepoPath)()
});

exports.builder = builder;

const runCheckTask = ({
  silent,
  debug,
  repoPath
}) => {
  return new _listr.default([{
    title: 'Check installations',
    task: ctx => {
      ctx.localBinPath = (0, _toolkit.getLocalBinary)('ipfs', process.cwd());
      ctx.globalBinPath = (0, _toolkit.getGlobalBinary)('ipfs');
    }
  }, {
    title: 'Check repository',
    task: async ctx => {
      ctx.repoExists = (0, _fs.existsSync)(repoPath);
      if (!ctx.repoExists) return;
      const [version, size, config] = await Promise.all([(0, _toolkit.getRepoVersion)(repoPath), (0, _toolkit.getRepoSize)(repoPath), (0, _toolkit.getRepoConfig)(repoPath)]);
      ctx.repoVersion = version;
      ctx.repoSize = size;
      ctx.peerID = (0, _toolkit.getPeerIDConfig)(config);
      ctx.daemonPorts = (0, _toolkit.getPorts)(config);
    }
  }, {
    title: 'Check the daemon',
    skip: ctx => !ctx.repoExists,
    task: async ctx => {
      ctx.daemonRunning = await (0, _toolkit.isLocalDaemonRunning)({
        protocol: 'http',
        host: '127.0.0.1',
        port: ctx.daemonPorts.api
      });
    }
  }, {
    title: 'Check CORS',
    skip: ctx => !ctx.daemonRunning,
    task: async ctx => {
      try {
        ctx.corsEnabled = await (0, _toolkit.isCorsConfigured)({
          protocol: 'http',
          host: '127.0.0.1',
          port: ctx.daemonPorts.api
        });
      } catch (err) {}
    }
  }, {
    title: 'Check MultiAddresses',
    skip: ctx => !ctx.daemonRunning,
    task: async ctx => {
      const [publicIP, internalIP] = await Promise.all([_publicIp.default.v4(), _internalIp.default.v4()]);
      ctx.publicSwarmMultiAddr = '/ip4/' + publicIP + '/tcp/4001/ipfs' + ctx.peerID;
      ctx.internalSwarmMultiAddr = '/ip4/' + internalIP + '/tcp/4001/ipfs' + ctx.peerID;
      ctx.localSwarmMultiAddr = '/ip4/' + '127.0.0.1' + '/tcp/4001/ipfs' + ctx.peerID;
    }
  } // Other possible checks:
  // Whether the gateway is open to the public ??
  // Whether the API is open to the public??
  // StorageMax
  ], (0, _listrOptions.default)(silent, debug)).run();
};

const handler = async argv => {
  const {
    reporter,
    debug,
    silent,
    repoPath
  } = argv;
  const {
    localBinPath,
    globalBinPath,
    repoVersion,
    repoSize,
    peerID,
    daemonPorts,
    daemonRunning,
    corsEnabled,
    publicSwarmMultiAddr,
    internalSwarmMultiAddr,
    localSwarmMultiAddr,
    repoExists
  } = await runCheckTask({
    silent,
    debug,
    repoPath
  });
  reporter.info(`Local installation: ${(0, _chalk.blue)(localBinPath || 'not installed')}`);
  reporter.info(`Global installation: ${(0, _chalk.blue)(globalBinPath || 'not installed')}`);
  reporter.newLine();

  if (repoExists) {
    reporter.info(`Repository location: ${(0, _chalk.blue)(repoPath)}`);
    reporter.info(`Repository version: ${(0, _chalk.blue)(repoVersion)}`);
    reporter.info(`Repository size: ${(0, _chalk.blue)(repoSize)}`);
    reporter.newLine();
    reporter.info(`API port: ${(0, _chalk.blue)(daemonPorts.api)}`);
    reporter.info(`Gateway port: ${(0, _chalk.blue)(daemonPorts.gateway)}`);
    reporter.info(`Swarm port: ${(0, _chalk.blue)(daemonPorts.swarm)}`);
    reporter.newLine();
    reporter.info(`PeerID: ${(0, _chalk.bgWhite)((0, _chalk.black)(peerID))}`);
    reporter.info(`Daemon: ${daemonRunning ? (0, _chalk.green)('running') : (0, _chalk.red)('stopped')}`);
  } else {
    reporter.info(`Repository: ${(0, _chalk.red)('uninitialized')}`);
  }

  if (daemonRunning) {
    reporter.info(`CORS: ${corsEnabled ? (0, _chalk.green)('enabled') : (0, _chalk.red)('disabled')}`);
    reporter.newLine();
    reporter.info(`Public Swarm MultiAddress: ${(0, _chalk.blue)(publicSwarmMultiAddr)}`);
    reporter.info(`Internal Swarm MultiAddress: ${(0, _chalk.blue)(internalSwarmMultiAddr)}`);
    reporter.info(`Local Swarm MultiAddress: ${(0, _chalk.blue)(localSwarmMultiAddr)}`);
  }
};

exports.handler = handler;
//# sourceMappingURL=status.js.map