"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.artifactExists = artifactExists;
exports.flattenCodeFileExists = flattenCodeFileExists;
exports.changeManifestForHttpServedFrom = changeManifestForHttpServedFrom;
exports.writeApplicationArtifact = writeApplicationArtifact;
exports.generateFlattenedCode = generateFlattenedCode;
exports.checkIfNewArticatIsIdentical = checkIfNewArticatIsIdentical;
exports.copyCurrentApplicationArtifacts = copyCurrentApplicationArtifacts;
exports.MissingFunctionsArtifacts = void 0;

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _web3Utils = require("web3-utils");

var _fsExtra = require("fs-extra");

var _toolkit = require("@aragon/toolkit");

var _util = require("../../util");

var _flattenCode = _interopRequireDefault(require("../../helpers/flattenCode"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

//
class MissingFunctionsArtifacts extends Error {}

exports.MissingFunctionsArtifacts = MissingFunctionsArtifacts;

const getRoles = roles => roles.map(role => Object.assign(role, {
  bytes: (0, _web3Utils.keccak256)(role.id)
}));
/**
 * Load environments from the arapp file
 * @param {string} cwd CWD
 * @return {Object[]} arapp environments
 */


function getEnvironments(cwd) {
  const arappManifestPath = _path.default.resolve(cwd, _toolkit.ARAPP_FILE);

  const arappManifestFile = (0, _fsExtra.readJsonSync)(arappManifestPath);
  return arappManifestFile.environments;
}
/**
 * Load main contract ABI from the build artifacts
 * @param {string} cwd CWD
 * @param {string} contractPath path to main contract
 * @return {Object[]} ABI
 */


function getContractAbi(cwd, contractPath) {
  const contractInterfacePath = _path.default.resolve(cwd, 'build/contracts', _path.default.basename(contractPath, '.sol') + '.json');

  const contractInterface = (0, _fsExtra.readJsonSync)(contractInterfacePath);
  return contractInterface.abi;
}
/**
 * Check if the artifact file exists at a given outputPath
 * @param {string} outputPath Output path
 * @return {boolean} Exists
 */


function artifactExists(outputPath) {
  const artifactPath = _path.default.resolve(outputPath, _toolkit.ARTIFACT_FILE);

  return (0, _fsExtra.pathExistsSync)(artifactPath);
}
/**
 * Load the artifact file from a given outputPath
 * @param {string} outputPath Output path
 * @return {Object} old artifact
 */


function getArtifact(outputPath) {
  const artifactPath = _path.default.resolve(outputPath, _toolkit.ARTIFACT_FILE);

  return (0, _fsExtra.readJsonSync)(artifactPath);
}
/**
 * Check if the flattenCode file exists at a given outputPath
 * @param {string} outputPath Output path
 * @return {boolean} exists
 */


function flattenCodeFileExists(outputPath) {
  const artifactPath = _path.default.resolve(outputPath, _toolkit.SOLIDITY_FILE);

  return (0, _fsExtra.pathExistsSync)(artifactPath);
}
/**
 * Change the manifest path's when served from HTTP
 * @param {string} httpServedFrom flag
 * @return {void} -
 */


function changeManifestForHttpServedFrom(httpServedFrom) {
  const projectRoot = (0, _util.findProjectRoot)();

  const manifestOrigin = _path.default.resolve(projectRoot, _toolkit.MANIFEST_FILE);

  const manifestDst = _path.default.resolve(httpServedFrom, _toolkit.MANIFEST_FILE);

  if (!(0, _fsExtra.pathExistsSync)(manifestDst) && (0, _fsExtra.pathExistsSync)(manifestOrigin)) {
    const manifest = (0, _fsExtra.readJsonSync)(manifestOrigin);
    (0, _fsExtra.writeJsonSync)(manifestDst, { ...manifest,
      start_url: _path.default.basename(manifest.start_url),
      script: _path.default.basename(manifest.script)
    });
  }
}
/**
 * Save artifact
 * @param {Object} artifact Artifact object to write
 * @param {string} outputPath Path to write the artifact to
 * @return {void} -
 */


function writeApplicationArtifact(artifact, outputPath) {
  (0, _fsExtra.writeJsonSync)(_path.default.resolve(outputPath, _toolkit.ARTIFACT_FILE), artifact, {
    spaces: '\t'
  });
}

async function generateFlattenedCode(dir, sourcePath) {
  const flattenedCode = await (0, _flattenCode.default)([sourcePath]);

  _fs.default.writeFileSync(_path.default.resolve(dir, _toolkit.SOLIDITY_FILE), flattenedCode);
}
/**
 * = Sanity check
 * Makes sure the new artifact components are identical to the currents
 * @param {string} cwd CWD
 * @param {Object[]} newRoles contract roles
 * @param {string} newContractPath path to main contract
 * @param {string|Object} oldArtifactOrOutputPath path to artifact of the artifact
 * @return {boolean} artifact is identical
 */


function checkIfNewArticatIsIdentical(cwd, newRoles, newContractPath, oldArtifactOrOutputPath) {
  const oldArtifact = typeof oldArtifactOrOutputPath === 'string' ? getArtifact(oldArtifactOrOutputPath) : oldArtifactOrOutputPath;
  const {
    roles,
    environments,
    abi,
    path
  } = oldArtifact;
  const newContractRoles = getRoles(newRoles);
  const newContractEnvironments = getEnvironments(cwd);
  const newContractAbi = getContractAbi(cwd, newContractPath);
  return JSON.stringify(newContractRoles) === JSON.stringify(roles) && JSON.stringify(newContractEnvironments) === JSON.stringify(environments) && JSON.stringify(newContractAbi) === JSON.stringify(abi) && newContractPath === path;
}

async function copyCurrentApplicationArtifacts(web3, cwd, outputPath, repo, newVersion, roles, contractPath, apmOptions) {
  const copyingFiles = [_toolkit.ARTIFACT_FILE, _toolkit.SOLIDITY_FILE];
  const {
    content
  } = repo;
  const uri = `${content.provider}:${content.location}`;
  const copyFilesData = await Promise.all(copyingFiles.map(async file => {
    try {
      return {
        filePath: _path.default.resolve(outputPath, file),
        fileContent: await (0, _toolkit.apmGetFile)(web3, uri, file, apmOptions),
        fileName: file
      };
    } catch (e) {
      // Only throw if fetching artifact fails, if code can't be found
      // continue as it could be fetched from previous versions
      if (file === _toolkit.ARTIFACT_FILE) {
        throw e;
      }
    }
  }));

  const updateArtifactVersion = (file, version) => {
    const newContent = JSON.parse(file.fileContent);
    newContent.version = version;
    return { ...file,
      fileContent: JSON.stringify(newContent, null, 2)
    };
  };

  copyFilesData.filter(item => item).map(file => {
    if (file.fileName === _toolkit.ARTIFACT_FILE) {
      const currentArtifactIsIdentical = checkIfNewArticatIsIdentical(cwd, roles, contractPath, JSON.parse(file.fileContent));

      if (!currentArtifactIsIdentical) {
        throw new Error('Artifact mismatch');
      } else {
        return updateArtifactVersion(file, newVersion);
      }
    }

    return file;
  }).forEach(({
    filePath,
    fileContent
  }) => _fs.default.writeFileSync(filePath, fileContent));
}
//# sourceMappingURL=generateArtifact.js.map