"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getPrevAndNextVersion = getPrevAndNextVersion;
exports.InvalidBump = void 0;

var _semver = _interopRequireDefault(require("semver"));

var _toolkit = require("@aragon/toolkit");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class InvalidBump extends Error {}
/**
 * @typedef {Object} PrevAndNextVersionReturn
 * @property {Object} [initialRepo]
 * @property {string} [prevVersion]
 * @property {string} [version]
 * @property {boolean} [shouldDeployContract]
 */

/**
 * Compute the next version and return previous repo and version
 * @param {string} appName "finance.aragonpm.eth"
 * @param {string} bumpOrVersion "minor" | "0.1.4"
 * @param {*} web3 todo
 * @param {*} apmOptions todo
 * @return {PrevAndNextVersionReturn} Multiple values, check JSDoc
 */


exports.InvalidBump = InvalidBump;

async function getPrevAndNextVersion(appName, bumpOrVersion, web3, apmOptions) {
  try {
    const initialRepo = await (0, _toolkit.getApmRepo)(web3, appName, apmOptions);
    const prevVersion = initialRepo.version;
    const version = resolveBumpOrVersion(bumpOrVersion, prevVersion);
    const isValid = await (0, _toolkit.apmIsValidBump)(web3, appName, prevVersion, version, apmOptions);
    if (!isValid) throw new InvalidBump();
    return {
      initialRepo,
      prevVersion,
      version,
      shouldDeployContract: _semver.default.major(prevVersion) !== _semver.default.major(version)
    };
  } catch (e) {
    if (e.message.includes('Invalid content URI')) {
      return {};
    } // Repo doesn't exist yet, deploy the first version


    const version = resolveBumpOrVersion(bumpOrVersion, '0.0.0');

    if (!_toolkit.APM_INITIAL_VERSIONS.includes(version)) {
      const validVersionsList = _toolkit.APM_INITIAL_VERSIONS.join(', ');

      throw Error(`Invalid initial version '${version}', valid values: ${validVersionsList}`);
    }

    return {
      version,
      shouldDeployContract: true // assume first version should deploy a contract

    };
  }
}
/**
 * Get bumped version
 * @param {string} bumpOrVersion "minor" | "0.1.4"
 * @param {string} prevVersion Version to bump from: "0.1.0"
 * @return {string} resulting sem version: "0.2.0"
 */


function resolveBumpOrVersion(bumpOrVersion, prevVersion) {
  return _semver.default.valid(bumpOrVersion) ? _semver.default.valid(bumpOrVersion) : _semver.default.inc(prevVersion, bumpOrVersion);
}
//# sourceMappingURL=getPrevAndNextVersion.js.map