"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.configEnvironment = configEnvironment;
exports.NoNetworkInTruffleConfig = exports.NoEnvironmentInDefaults = exports.NoEnvironmentInArapp = void 0;

var _web = _interopRequireDefault(require("web3"));

var _url = _interopRequireDefault(require("url"));

var _environments = _interopRequireDefault(require("../../../config/environments.default"));

var _truffle = _interopRequireDefault(require("../../../config/truffle.default"));

var _constants = require("../../commands/devchain_cmds/utils/constants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

//
const FRAME_ENDPOINT = 'ws://localhost:1248';
const FRAME_ORIGIN = 'aragonCLI';
const ARAGON_RINKEBY_ENDPOINT = 'wss://rinkeby.eth.aragon.network/ws';
const ARAGON_MAINNET_ENDPOINT = 'wss://mainnet.eth.aragon.network/ws';
const IPFS_RPC = 'http://localhost:5001#default';
const IPFS_GATEWAY = 'http://localhost:8080/ipfs';

class NoEnvironmentInArapp extends Error {}

exports.NoEnvironmentInArapp = NoEnvironmentInArapp;

class NoEnvironmentInDefaults extends Error {}

exports.NoEnvironmentInDefaults = NoEnvironmentInDefaults;

class NoNetworkInTruffleConfig extends Error {}

exports.NoNetworkInTruffleConfig = NoNetworkInTruffleConfig;

function getEnv(arapp, environment) {
  if (arapp) {
    if (!environment) environment = 'default';
    const env = arapp.environments[environment];
    if (!env) throw new NoEnvironmentInArapp(environment);
    return env;
  } else {
    // if there is no arapp.json and the command is not init default to the "global" config
    // designed for the dao commands including dao acl
    if (!environment) environment = 'aragon:local';
    const env = _environments.default[environment];
    if (!env) throw new NoEnvironmentInDefaults(environment);
    return env;
  }
}

const configureAPM = (ipfsRPC, gateway, ensRegistryAddress) => {
  const rpc = {};

  if (ipfsRPC) {
    const uri = new _url.default.URL(ipfsRPC);
    Object.assign(rpc, {
      protocol: uri.protocol.replace(':', ''),
      host: uri.hostname,
      port: parseInt(uri.port)
    });

    if (uri.hash === '#default') {
      rpc.default = true;
    }
  }

  return {
    ensRegistryAddress,
    ipfs: {
      rpc,
      gateway
    }
  };
};

function configureNetwork(network, truffleConfig, options) {
  const {
    useFrame,
    ignoreNetwork
  } = options || {};
  if (ignoreNetwork) return {};

  if (useFrame) {
    return {
      name: `frame-${network}`,
      provider: new _web.default.providers.WebsocketProvider(FRAME_ENDPOINT, {
        headers: {
          origin: FRAME_ORIGIN
        }
      } // Provider options
      )
    };
  }

  const truffleNetwork = truffleConfig.networks[network];

  if (!truffleNetwork) {
    throw new NoNetworkInTruffleConfig(network);
  }

  const {
    provider,
    host,
    port
  } = truffleNetwork;

  const getProvider = provider => {
    if (provider) {
      if (typeof provider === 'function') return provider();else return provider;
    } else if (host && port) {
      return new _web.default.providers.WebsocketProvider(`ws://${host}:${port}`);
    } else {
      return new _web.default.providers.HttpProvider('http://localhost:8545');
    }
  };

  return { ...truffleNetwork,
    name: network,
    provider: getProvider(provider)
  };
}

function configEnvironment({
  ignoreNetwork,
  useFrame,
  environment,
  wsRpc,
  ensRegistry,
  ipfsRpc,
  ipfsGateway,
  arapp,
  truffleConfig = _truffle.default
}) {
  const networkOptions = {
    ignoreNetwork,
    useFrame
  }; // default environment (no arapp.json) uses different naming

  const env = getEnv(arapp, environment); // (Todo): Are these mutations necessary?

  if (arapp) {
    arapp.appName = env.appName;
    arapp.environments = {
      [environment || 'default']: env
    }; // only include the selected environment in the module
  }

  const wsProviderUrl = env.wsRPC ? env.wsRPC : env.network === 'rinkeby' ? ARAGON_RINKEBY_ENDPOINT : env.network === 'mainnet' ? ARAGON_MAINNET_ENDPOINT : null;
  return {
    apm: configureAPM(ipfsRpc || IPFS_RPC, ipfsGateway || env.apm && env.apm.ipfs.gateway || IPFS_GATEWAY, ensRegistry || env.registry || _constants.DEVCHAIN_ENS),
    network: configureNetwork(env.network, truffleConfig, networkOptions),
    wsProvider: wsProviderUrl ? new _web.default.providers.WebsocketProvider(wsRpc || wsProviderUrl) : null,
    arapp
  };
}
//# sourceMappingURL=configEnvironment.js.map