"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.normalizeOutput = normalizeOutput;
exports.isHttpServerOpen = isHttpServerOpen;
exports.askForConfirmation = exports.askForChoice = exports.askForInput = exports.parseArgumentStringIfPossible = exports.parseAsArray = exports.parseAsBoolean = exports.runScriptHelper = exports.installDeps = exports.findProjectRoot = void 0;

var _findUp = _interopRequireDefault(require("find-up"));

var _path = _interopRequireDefault(require("path"));

var _execa = _interopRequireDefault(require("execa"));

var _fs = _interopRequireDefault(require("fs"));

var _os = _interopRequireDefault(require("os"));

var _fsExtra = require("fs-extra");

var _http = require("http");

var _inquirer = _interopRequireDefault(require("inquirer"));

var _toolkit = require("@aragon/toolkit");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let cachedProjectRoot;
/**
 * Some characters are rendered differently depending on the OS.
 *
 * @param {string} stdout
 */

function normalizeOutput(stdout) {
  const next = stdout // remove user-specific paths
  .replace(/❯/g, '>').replace(/ℹ/g, 'i') // TODO: remove after https://github.com/aragon/aragon-cli/issues/367 is fixed
  .replace(/cli.js/g, 'aragon').replace(/index.js/g, 'aragon') // replace homedir in paths
  .replace(new RegExp(_os.default.homedir(), 'g'), '~') // sometimes there's an extra LF
  .trim();
  return next;
}

const findProjectRoot = () => {
  if (!cachedProjectRoot) {
    try {
      cachedProjectRoot = _path.default.dirname(_findUp.default.sync('arapp.json'));
    } catch (_) {
      throw new Error('This directory is not an Aragon project');
    }
  }

  return cachedProjectRoot;
};
/**
 * Check if an http server is listening at a given url
 * @param {string} url Server url
 * @returns {boolean} true if server is returning a valid response
 */


exports.findProjectRoot = findProjectRoot;

function isHttpServerOpen(url) {
  return new Promise(resolve => {
    (0, _http.request)(url, {
      method: 'HEAD'
    }, r => {
      resolve(r.statusCode >= 200 && r.statusCode < 400);
    }).on('error', () => resolve(false)).end();
  });
}

const installDeps = (cwd, task) => {
  const bin = (0, _toolkit.getNodePackageManager)();
  const installTask = (0, _execa.default)(bin, ['install'], {
    cwd
  });
  installTask.stdout.on('data', log => {
    if (!log) return;
    task.output = log;
  });
  return installTask.catch(err => {
    throw new Error(`${err.message}\n${err.stderr}\n\nFailed to install dependencies. See above output.`);
  });
};
/**
 * Run a npm script.
 * @param {string} scriptName
 * @return {Promise<string>} feeback, if the helper aborts early it may return
 * a string with a reason message
 */


exports.installDeps = installDeps;

const runScriptHelper = async (scriptName, logCallback) => {
  if (!_fs.default.existsSync('package.json')) {
    return 'No package.json found';
  }

  const {
    scripts = {}
  } = await (0, _fsExtra.readJson)('package.json');

  if (!scripts[scriptName]) {
    return `${scriptName} script not defined in package.json`;
  }

  const bin = (0, _toolkit.getNodePackageManager)();
  const scriptProcess = (0, _execa.default)(bin, ['run', scriptName]);
  scriptProcess.stdout.on('data', log => {
    if (logCallback) logCallback(`npm run ${scriptName}: ${log}`);
  });

  try {
    await scriptProcess;
  } catch (err) {
    throw new Error(`${err.message}\n${err.stderr}\n\nScript ${scriptName} failed. See above output.`);
  }
};
/**
 * Parse a String to Boolean, or throw an error.
 *
 * The check is **case insensitive**! (Passing `"TRue"` will return `true`)
 *
 * @param {string} target must be a string
 * @returns {boolean} the parsed value
 */


exports.runScriptHelper = runScriptHelper;

const parseAsBoolean = target => {
  if (typeof target !== 'string') {
    throw new Error(`Expected ${target} to be of type string, not ${typeof target}`);
  }

  const lowercase = target.toLowerCase();

  if (lowercase === 'true') {
    return true;
  }

  if (lowercase === 'false') {
    return false;
  }

  throw new Error(`Cannot parse ${target} as boolean`);
};
/**
 * Parse a String to Array, or throw an error.
 *
 * @param {string} target must be a string
 * @returns {Array} the parsed value
 */


exports.parseAsBoolean = parseAsBoolean;

const parseAsArray = target => {
  if (typeof target !== 'string') {
    throw new Error(`Expected ${target} to be of type string, not ${typeof target}`);
  } // https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/JSON/parse


  const json = JSON.parse(target);

  if (Array.isArray(json)) {
    return json;
  }

  throw new Error(`Cannot parse ${target} as array`);
};
/**
 * Parse a String to Boolean or Array, or throw an error.
 *
 * @param {string} target must be a string
 * @returns {boolean|Array} the parsed value
 */


exports.parseAsArray = parseAsArray;

const parseArgumentStringIfPossible = target => {
  // convert to boolean: 'false' to false
  try {
    return parseAsBoolean(target);
  } catch (e) {} // convert to array: '["hello", 1, "true"]' to ["hello", 1, "true"]
  // TODO convert children as well ??


  try {
    return parseAsArray(target);
  } catch (e) {} // nothing to parse


  return target;
};

exports.parseArgumentStringIfPossible = parseArgumentStringIfPossible;

const askForInput = async message => {
  const {
    reply
  } = await _inquirer.default.prompt([{
    type: 'input',
    name: 'reply',
    message
  }]);
  return reply;
};

exports.askForInput = askForInput;

const askForChoice = async (message, choices) => {
  const {
    reply
  } = await _inquirer.default.prompt([{
    type: 'list',
    name: 'reply',
    message,
    choices
  }]);
  return reply;
};

exports.askForChoice = askForChoice;

const askForConfirmation = async message => {
  const {
    reply
  } = await _inquirer.default.prompt([{
    type: 'confirm',
    name: 'reply',
    message
  }]);
  return reply;
};

exports.askForConfirmation = askForConfirmation;
//# sourceMappingURL=util.js.map